# Mail in tool for the CMF

from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import UniqueObject
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.DirectoryView import createDirectoryView
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
import string, rfc822, multifile, StringIO, mimetools, re, os

mailin_globals = {}

class MailInTool (UniqueObject, SimpleItem):
    """ a mail in tool for handling mail coming in to the portal """
    id = 'portal_mailin'
    meta_type= 'CMF Mail In Tool'
    security = ClassSecurityInfo()

    manage_options = ( ({ 'label' : 'Overview', 'action' : 'manage_overview' }
                     ,  { 'label' : 'Configure', 'action' : 'manage_configure' }
                     ,
                     ) + SimpleItem.manage_options
                     )
    
    #
    #   ZMI methods
    #
    security.declareProtected( CMFCorePermissions.ManagePortal ,
                               'manage_overview' )
    manage_overview = PageTemplateFile('www/explainMailInTool', globals(),
                               __name__='manage_overview')
    
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'manage_configure' )    
    manage_configure = PageTemplateFile('www/configureMailInTool', globals(),
                                   __name__='manage_configure')    
    
   
    def __init__(self): 
        self.method = 'mail_received'
    
    security.declarePublic('setup_skins')
    def setup_skins(self):
        """ Sets up the mailin_tool skin """
        skinstool=getToolByName(self, 'portal_skins')
        try:
            createDirectoryView( skinstool, os.path.join('Products','CMFMailIn','skins','mailin')) 
        except:
            pass
        
        skinpaths = skinstool.getSkinPaths()
        for skin in skinpaths:
            paths = map(string.strip, string.split(skin[1],','))
            if not 'mailin' in paths:
                paths.append('mailin')
                skinstool.addSkinSelection(skin[0], string.join( paths, ', '))
        
        # add the content type to the portal_types tool
        typestool=getToolByName(self, 'portal_types')
        typestool.manage_addTypeInformation(id=None, add_meta_type="Factory-based Type Information", typeinfo_name="CMFMailIn: CMF MailMessage")
                
        return "The mailin skins have been setup"
    
    security.declareProtected( CMFCorePermissions.ManagePortal
                             , 'edit_configuration' )
    def edit_configuration(self, method):
        """ Change the configuration of the mailin tool """
        self.method = method
        if hasattr(self.REQUEST, 'RESPONSE'):
            self.REQUEST.RESPONSE.redirect('manage_configure')

    security.declarePublic('getMethodName')
    def getMethodName(self):
        """ Returns the name of the method called when mail arrives """
        return self.method
    
    security.declarePublic( 'postMailMessage' )
    def postMailMessage(self, file=None):
        """
        This method will create a dictionary containing all the information about the Mail Message
        The dictionary is then handed to the method stated in self.method for processing
        """
       
        if not file:
            raise IOError, 'No Mail Message Supplied'
        
        sio = StringIO.StringIO(file)  # Put it in a StringIO so we can play with it over and over
        m = rfc822.Message(sio)  # Render the message through rfc822 so we can get the headers
        sio.seek(0)

        theMail = {}  # A New Dictionary
        
        # Place all the email header in the headers dictionary in theMail
        headers = theMail['headers'] = {}
        for (name,value) in m.items():
            headers[name]=value
        
        # put localpart in theMail
        address_to = headers.get('envelope-to',headers.get('to',''))
        if not address_to:
            raise KeyError,'neither envelope-to or to found in supplied the mail message header'

        theMail['localpart'] = self._getLocalPart(address_to)
         
        # Process mime encoded mail messages
        mf = multifile.MultiFile(sio,0)
        theMail['attachments'] = {}
        self._unpack_part(theMail,mf,1)

        try:
            portal_url = self.portal_url.getPortalPath()
            if portal_url[-1]!='/': portal_url=portal_url+'/'
        except:
            portal_url = ''
            
        if self.method:
            return self.restrictedTraverse(portal_url+self.method)(theMail=theMail)
        
        self.REQUEST.RESPONSE.notFoundError('MailIn method not specified')


    security.declarePrivate( '_unpack_part' )
    def _unpack_part(self,theMail, file, partno):
        # Will unpack a multipart message in the MIME format and enter it in to
        # the dictionary theMail

        headers = mimetools.Message(file)
        type = headers.getmaintype()
        if type == 'multipart':
            boundary = headers.getparam("boundary")
            file.push(boundary)
            file.readlines()
            while not file.last:
                file.next()
                partno = partno + 1
                self._unpack_part(theMail, file, partno)
            file.pop()
            return

        name = headers.getparam("name")
        subtype = headers.getsubtype()
        if type=='text' and subtype=='plain' and not name:
            # overwrite the body so that we get _only_ the body text.
            # (Uses the plain version of the body in all situations.
            #  HTML formated body will be in the attachments dictionary)
            theMail['body'] = file.read()
        else:
            if not name:
                name = 'part%d' % (partno,)
                name = name + '-'+subtype

            encoding = headers.getencoding()


            outfile = StringIO.StringIO()
            if encoding == '7bit':
                mimetools.copyliteral(file,outfile)
            else:
                mimetools.decode(file,outfile,encoding)

            theMail['attachments'][name]=outfile.getvalue()
            outfile.close()

    def _getLocalPart(self, address):
        str1 = address[:string.find(address,'@')]
        if string.find(str1,'<')>-1:
            return str1[string.find(str1,'<')+1:]
        return str1            
            
InitializeClass(MailInTool)    